from dataclasses import asdict
from typing import Union, Optional
from aiohttp import request, BasicAuth

from pc4store.data import (
    CreateOrderInput, CreateOrderSuccess, CreateOrderError, OrderData, Transfer,
    TransferInput, TransferSuccess, TransferError
)
from .base import BaseClient


class Pc4StoreAsyncClient(BaseClient):
    async def create_order(
            self,
            params: Union[CreateOrderInput, dict]
    ) -> Union[CreateOrderSuccess, CreateOrderError]:
        params = self._get_validated_create_order_input(params)
        async with request('POST', self.CREATE_ORDER,
                           auth=BasicAuth(self.store_id, self.store_key),
                           json=asdict(params)
                           ) as response:
            try:
                data = await response.json()
                return self._get_formatted_create_order_res(data)
            except Exception as e:
                return CreateOrderError(error=str(e))

    async def get_order(self, order_id: str) -> Optional[OrderData]:
        async with request('GET', f'{self.GET_ORDER}/{order_id}',
                           auth=BasicAuth(self.store_id, self.store_key),
                           ) as response:
            data = await response.json()
            order_dict = data.pop('payload', {}).pop('order', None)
            return OrderData.from_dict(order_dict) if order_dict else None

    async def transfer(
            self,
            params: Union[TransferInput, dict]
    ) -> Union[TransferSuccess, TransferError]:
        params = self._get_validated_transfer_input(params)
        async with request('POST', self.TRANSFER,
                           auth=BasicAuth(self.store_id, self.store_key),
                           json=asdict(params)
                           ) as response:
            try:
                data = await response.json()
                return self._get_formatted_transfer_res(data)
            except Exception as e:
                return TransferError(error=str(e))

    async def get_transfer(self, transfer_id: str) -> Optional[Transfer]:
        async with request('GET', f'{self.GET_TRANSFER}/{transfer_id}',
                           auth=BasicAuth(self.store_id, self.store_key),
                           ) as response:
            data = await response.json()
            transfer_dict = data.pop('payload', {}).pop('transfer', None)
            return Transfer.from_dict(transfer_dict) if transfer_dict else None



from multidict import MultiDict

from pc4store.clients import Pc4StoreAsyncClient

my_store_id = 'e40e58c5-2dd2-4c3b-994f-5af9c7ea255b'
my_store_secret_key = 'b8c4d4c6-579a-41aa-b887-d390242ce339'

# initialize client
client = Pc4StoreAsyncClient(store_id=my_store_id,
                             store_key=my_store_secret_key)

request = MultiDict()  # callback request that you receive from PC4Store API

if client.is_signature_correct(request.json(), request.headers):
    # you can trust this callback
    ...
else:
    # it could be cheating
    ...



from pc4store.clients import Pc4StoreClient
from pc4store.data import CreateOrderInput, OrderData

my_store_id = 'e40e58c5-2dd2-4c3b-994f-5af9c7ea255b'
my_store_secret_key = 'b8c4d4c6-579a-41aa-b887-d390242ce339'

# initialize client
client = Pc4StoreClient(store_id=my_store_id, store_key=my_store_secret_key)

# create order
res = client.create_order(CreateOrderInput(
    currency_name='USDCASH',
    currency_smart_contract='token.pcash',
    amount_to_pay='99.99999',
    response_url='https://api.my-store/payment-callback/',
    expiration_time=30 * 60,  # 30 min
    merchant_order_id='12345',
    description='optional description for this order',
    success_payment_redirect_url='https://my-store/orders/12345/success',
    failed_payment_redirect_url='https://my-store/orders/12345/failed',
))

if res.status == 'OK':
    order: OrderData = res.payload.order  # all data about order
    payment_link = order.payment_url  # link to redirect user for payment
    order_id = order.id  # unique order id in payment system
else:
    print(res.error)



import hashlib
import json
from typing import Union

from pc4store import config

from pc4store.data import (
    CreateOrderInput, CreateOrderSuccess, CreateOrderError, TransferInput,
    TransferSuccess, TransferError
)
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PublicKey


class BaseClient:
    CREATE_ORDER = f'{config.HOST}/create'
    GET_ORDER = f'{config.HOST}/order_info'
    TRANSFER = f'{config.HOST}/transfer'
    GET_TRANSFER = f'{config.HOST}/transfer_info'

    def __init__(self, store_id: str, store_key: str):
        self.store_id = store_id
        self.store_key = store_key
        self.public_key: Ed25519PublicKey = Ed25519PublicKey.from_public_bytes(
            bytes(bytearray.fromhex(config.PUBLIC_KEY)))

    @classmethod
    def _get_validated_create_order_input(cls,
                                          params: Union[CreateOrderInput, dict]
                                          ) -> CreateOrderInput:
        assert isinstance(params, (CreateOrderInput, dict))
        if isinstance(params, dict):
            return CreateOrderInput.from_dict(params)
        else:
            return params

    @classmethod
    def _get_formatted_create_order_res(
            cls, data: dict
    ) -> Union[CreateOrderSuccess, CreateOrderError]:
        if data['status'] == 'OK':
            return CreateOrderSuccess.from_dict(data)
        else:
            return CreateOrderError(error=data.get('error'))

    @classmethod
    def _get_validated_transfer_input(cls, params: Union[TransferInput, dict]
                                      ) -> TransferInput:
        assert isinstance(params, (TransferInput, dict))
        if isinstance(params, dict):
            return TransferInput.from_dict(params)
        else:
            return params

    @classmethod
    def _get_formatted_transfer_res(
            cls, data: dict
    ) -> Union[TransferSuccess, TransferError]:
        if data['status'] == 'OK':
            return TransferSuccess.from_dict(data)
        else:
            return TransferError(error=data.get('error'))

    def is_signature_correct(self, json_body: dict, headers: dict) -> bool:
        signature = headers.get('SIGNATURE')
        assert signature is not None
        str_body = json.dumps(json_body, separators=(',', ':'))
        hash_body = hashlib.sha256(str_body.encode()).hexdigest()
        try:
            self.public_key.verify(bytes(bytearray.fromhex(signature)),
                                   bytes(bytearray.fromhex(hash_body)))
        except Exception:
            return False
        return True



from pc4store.clients import Pc4StoreAsyncClient
from pc4store.data import OrderData

my_store_id = 'e40e58c5-2dd2-4c3b-994f-5af9c7ea255b'
my_store_secret_key = 'b8c4d4c6-579a-41aa-b887-d390242ce339'

# initialize client
client = Pc4StoreAsyncClient(store_id=my_store_id,
                             store_key=my_store_secret_key)

order_id = 'order_id_in_payment_system'

order: OrderData = await client.get_order(order_id)

if order:
    # do some staff
    print('Actual order status: ', order.status)
else:
    print('Opps, order not found...')



import random
from decimal import Decimal
from string import ascii_lowercase, digits

CURRENCIES = (
    ('UAHCASH', 'cash.token'),
    ('RUBCASH', 'cash.token'),
    ('USDCASH', 'cash.token')
)


def rand_str(len_=7):
    alphabet = list(ascii_lowercase + digits)
    random.shuffle(alphabet)
    return ''.join(alphabet[:len_])


def rand_letters(len_=7):
    alphabet = list(ascii_lowercase)
    random.shuffle(alphabet)
    return ''.join(alphabet[:len_])


def rand_amount_in_coins(min_value=0, max_value=1000000000):
    return random.randint(min_value, max_value)


def rand_amount(min_value: Decimal = Decimal(0),
                max_value: Decimal = Decimal(10000),
                prec: int = 5):
    amount_in_coins = rand_amount_in_coins(int(min_value * 10 ** prec),
                                           int(max_value * 10 ** prec))
    return Decimal(amount_in_coins) / Decimal(10 ** prec)



from dataclasses import asdict
from typing import Union, Optional
import requests

from pc4store.data import (
    CreateOrderInput, CreateOrderSuccess, CreateOrderError, OrderData, Transfer,
    TransferInput, TransferSuccess, TransferError
)
from .base import BaseClient


class Pc4StoreClient(BaseClient):
    def create_order(
            self,
            params: Union[CreateOrderInput, dict]
    ) -> Union[CreateOrderSuccess, CreateOrderError]:
        params = self._get_validated_create_order_input(params)
        response = requests.post(self.CREATE_ORDER,
                                 auth=(self.store_id, self.store_key),
                                 json=asdict(params))
        try:
            data = response.json()
            return self._get_formatted_create_order_res(data)
        except Exception as e:
            return CreateOrderError(error=str(e))

    def get_order(self, order_id: str) -> Optional[OrderData]:
        response = requests.get(f'{self.GET_ORDER}/{order_id}',
                                auth=(self.store_id, self.store_key))
        data = response.json()
        order_dict = data.pop('payload', {}).pop('order', None)
        return OrderData.from_dict(order_dict) if order_dict else None

    def transfer(
            self,
            params: Union[TransferInput, dict]
    ) -> Union[TransferSuccess, TransferError]:
        params = self._get_validated_transfer_input(params)
        response = requests.post(self.TRANSFER,
                                 auth=(self.store_id, self.store_key),
                                 json=asdict(params))
        try:
            data = response.json()
            return self._get_formatted_transfer_res(data)
        except Exception as e:
            return TransferError(error=str(e))

    def get_transfer(self, transfer_id: str) -> Optional[Transfer]:
        response = requests.get(f'{self.GET_TRANSFER}/{transfer_id}',
                                auth=(self.store_id, self.store_key))
        data = response.json()
        transfer_dict = data.pop('payload', {}).pop('transfer', None)
        return Transfer.from_dict(transfer_dict) if transfer_dict else None



import dataclasses
from dataclasses import dataclass, fields
from datetime import datetime
from enum import Enum


class FromDictMixin:
    @classmethod
    def from_dict(cls, input_: dict):
        expected_fields = cls.__dataclass_fields__.items()
        required_fields = [field for field, type_ in expected_fields if
                           type_.default == dataclasses.MISSING]
        for field in required_fields:
            assert input_.get(
                field) is not None, f'Missing required field {field}'
        unexpected = set(input_) - set([field for field, _ in expected_fields])
        assert not unexpected, f'Got unexpected params: {list(unexpected)}'
        return cls(**input_)


@dataclass
class CreateOrderInput(FromDictMixin):
    currency_name: str
    currency_smart_contract: str
    amount_to_pay: str
    response_url: str
    expiration_time: int = None
    merchant_order_id: str = None
    description: str = None
    success_payment_redirect_url: str = None
    failed_payment_redirect_url: str = None


@dataclass
class TransferInput(FromDictMixin):
    amount: str
    currency_name: str
    currency_smart_contract: str
    eos_account: str
    response_url: str = None


class OrderStatus(Enum):
    CREATED = 'CREATED'
    MONEY_RECEIVED = 'MONEY_RECEIVED'
    PAID = 'PAID'
    EXPIRED = 'EXPIRED'
    CANCELLED = 'CANCELLED'


class TxnType(Enum):
    ORDER_PAYMENT = 'ORDER_PAYMENT'
    PAYBACK = 'PAYBACK'
    WITHDRAW = 'WITHDRAW'


class TxnStatus(Enum):
    INITIATED = 'INITIATED'  # initiated payback or withdraw
    SENDED = 'SENDED'  # sended payback or withdraw
    RECEIVED = 'RECEIVED'  # txn found and start proccess
    ACCEPTED = 'ACCEPTED'  # txn processed with success


@dataclass
class Amount:
    full_amount: str
    amount_after_tax: str
    fee: str

    @classmethod
    def from_dict(cls, data: dict):
        return cls(**data)


@dataclass
class Currency:
    name: str
    smart_contract: str
    precission: int

    @classmethod
    def from_dict(cls, data: dict):
        return cls(**data)


@dataclass
class OrderDetails:
    merchant_order_id: str = None
    description: str = None

    @classmethod
    def from_dict(cls, data: dict):
        return cls(**data)


@dataclass
class Action:
    txid: str
    block_number: str
    global_sequence: str
    is_irreversible: bool
    is_reversed: bool

    @classmethod
    def from_dict(cls, data: dict):
        return cls(**data)


@dataclass
class Transfer:
    id: str
    amount: Amount
    currency: Currency
    sender: str
    receiver: str
    txn_type: TxnType
    status: TxnStatus
    action: Action
    memo: str = ''

    @classmethod
    def from_dict(cls, data: dict):
        return cls(amount=Amount.from_dict(data.pop('amount')),
                   currency=Currency.from_dict(data.pop('currency')),
                   action=Action.from_dict(data.pop('action')),
                   **data)


@dataclass
class OrderData:
    id: str
    sequent_number: str
    amount: Amount
    currency: Currency
    status: OrderStatus
    expiration_date: datetime
    payment_url: str
    response_url: str
    is_test: bool
    details: OrderDetails
    success_payment_redirect_url: str = None
    failed_payment_redirect_url: str = None
    payment_transfer: Transfer = None

    @classmethod
    def from_dict(cls, data: dict):
        payment_transfer = Transfer.from_dict(
            data.pop('payment_transfer')
        ) if 'payment_transfer' in data else None
        return cls(amount=Amount.from_dict(data.pop('amount')),
                   currency=Currency.from_dict(data.pop('currency')),
                   payment_transfer=payment_transfer,
                   **data)


@dataclass
class OrderPayload:
    order: OrderData

    @classmethod
    def from_dict(cls, data: dict):
        return cls(order=OrderData.from_dict(data['order']))


@dataclass
class TransferPayload:
    transfer: Transfer

    @classmethod
    def from_dict(cls, data: dict):
        return cls(transfer=Transfer.from_dict(data['transfer']))


@dataclass
class ErrorResponse:
    error: str
    status = 'ERROR'


@dataclass
class CreateOrderSuccess:
    payload: OrderPayload
    status = 'OK'

    @classmethod
    def from_dict(cls, data: dict):
        return cls(payload=OrderPayload.from_dict(data['payload']))


@dataclass
class CreateOrderError(ErrorResponse):
    ...


@dataclass
class TransferSuccess:
    payload: TransferPayload
    status = 'OK'

    @classmethod
    def from_dict(cls, data: dict):
        return cls(payload=TransferPayload.from_dict(data['payload']))


@dataclass
class TransferError(ErrorResponse):
    ...



from setuptools import setup, find_packages

install_requires = ['requests>=2.25.1', 'cryptography>=3.4.5', 'aiohttp>=3.6.2']

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='pc4store',
    version='1.1.0',
    description='Python lib for integration with pc4store payment system',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages(exclude=['tests']),
    install_requires=install_requires,
    include_package_data=True,
)
